@@ -13,13 +13,9 @@ import zigpy_znp.config as conf
 import zigpy_znp.logger as log
 import zigpy_znp.commands as c
 from zigpy_znp import uart
-from zigpy_znp.types import nvids
+from zigpy_znp.nvram import NVRAMHelper
 from zigpy_znp.frames import GeneralFrame
-from zigpy_znp.exceptions import (
-    SecurityError,
-    CommandNotRecognized,
-    InvalidCommandResponse,
-)
+from zigpy_znp.exceptions import CommandNotRecognized, InvalidCommandResponse
 
 LOGGER = logging.getLogger(__name__)
 AFTER_CONNECT_DELAY = 1  # seconds
@@ -178,6 +174,8 @@ class ZNP:
         self._listeners = defaultdict(list)
         self._sync_request_lock = asyncio.Lock()
 
+        self.nvram = NVRAMHelper(self)
+
     def set_application(self, app):
         assert self._app is None
         self._app = app
@@ -186,7 +184,7 @@ class ZNP:
     def _port_path(self) -> str:
         return self._config[conf.CONF_DEVICE][conf.CONF_DEVICE_PATH]
 
-    async def connect(self, *, test_port=True, check_version=True) -> None:
+    async def connect(self, *, test_port=True) -> None:
         """
         Connects to the device specified by the "device" section of the config dict.
 
@@ -233,18 +231,7 @@ class ZNP:
                 LOGGER.debug("Testing connection to %s", self._port_path)
 
                 # Make sure that our port works
-                version = await self.request(c.SYS.Version.Req())
-
-                if (version.TransportRev, version.MajorRel, version.MinorRel) not in {
-                    (2, 2, 6),
-                    (2, 2, 7),
-                }:
-                    raise RuntimeError(
-                        f"Your device is running an unsupported"
-                        f" release of Z-Stack: {version}"
-                    )
-
-                self._version = version
+                self._version = await self.request(c.SYS.Version.Req())
                 self._capabilities = (await self.request(c.SYS.Ping.Req())).Capabilities
         except Exception:
             LOGGER.debug("Connection to %s failed, cleaning up", self._port_path)
@@ -490,16 +477,16 @@ class ZNP:
                 f"will have no effect"
             )
 
-        LOGGER.debug("Sending request: %s", request)
-
-        # If our request has no response, we cannot wait for one
-        if not request.Rsp:
-            LOGGER.debug("Request has no response, not waiting for one.")
-            self._uart.send(request.to_frame())
-            return
-
         # We should only be sending one SREQ at a time, according to the spec
         async with self._sync_request_lock:
+            LOGGER.debug("Sending request: %s", request)
+
+            # If our request has no response, we cannot wait for one
+            if not request.Rsp:
+                LOGGER.debug("Request has no response, not waiting for one.")
+                self._uart.send(request.to_frame())
+                return
+
             # We need to create the response listener before we send the request
             response_future = self.wait_for_responses(
                 [
@@ -555,140 +542,3 @@ class ZNP:
 
             async with async_timeout.timeout(timeout):
                 return await callback_rsp
-
-    async def nvram_delete(self, nv_id: nvids.BaseNvIds) -> bool:
-        """
-        Deletes an item from NVRAM. Returns whether or not the item existed.
-        """
-
-        length = (await self.request(c.SYS.OSALNVLength.Req(Id=nv_id))).ItemLen
-
-        if length == 0:
-            return False
-
-        delete_rsp = await self.request(
-            c.SYS.OSALNVDelete.Req(Id=nv_id, ItemLen=length)
-        )
-
-        return delete_rsp.Status == t.Status.SUCCESS
-
-    async def nvram_write(self, nv_id: nvids.BaseNvIds, value, *, create: bool = False):
-        """
-        Writes a complete value to NVRAM, optionally resizing and creating the item if
-        necessary.
-
-        Serializes all serializable values and passes bytes directly.
-        """
-
-        if hasattr(value, "serialize"):
-            value = value.serialize()
-        elif not isinstance(value, (bytes, bytearray)):
-            raise TypeError(
-                f"Only bytes or serializable types can be written to NVRAM."
-                f" Got {nv_id!r}={value!r} (type {type(value)})"
-            )
-
-        if not value:
-            raise ValueError(f"NV item {nv_id!r} cannot be empty")
-
-        length = (await self.request(c.SYS.OSALNVLength.Req(Id=nv_id))).ItemLen
-
-        # Recreate the item if the length is not correct
-
-        # XXX: Some NVIDs don't really exist and Z-Stack doesn't behave consistently
-        #      when operations are performed on them.
-        if length != len(value) and nv_id != t.nvids.NwkNvIds.POLL_RATE_OLD16:
-            if not create:
-                if length == 0:
-                    raise KeyError(f"NV item does not exist: {nv_id!r}")
-                else:
-                    raise ValueError(
-                        f"Stored length and actual length differ:"
-                        f" {length} != {len(value)}"
-                    )
-
-            if length != 0:
-                await self.request(
-                    c.SYS.OSALNVDelete.Req(Id=nv_id, ItemLen=length),
-                    RspStatus=t.Status.SUCCESS,
-                )
-
-            await self.request(
-                c.SYS.OSALNVItemInit.Req(
-                    Id=nv_id,
-                    ItemLen=len(value),
-                    Value=t.ShortBytes(value[:244]),
-                ),
-                RspStatus=t.Status.NV_ITEM_UNINIT,
-            )
-
-        # 244 bytes is the most you can fit in a single `SYS.OSALNVWriteExt` command
-        for offset in range(0, len(value), 244):
-            await self.request(
-                c.SYS.OSALNVWriteExt.Req(
-                    Id=nv_id,
-                    Offset=offset,
-                    Value=t.ShortBytes(value[offset : offset + 244]),
-                ),
-                RspStatus=t.Status.SUCCESS,
-            )
-
-    async def nvram_read(self, nv_id: nvids.BaseNvIds) -> bytes:
-        """
-        Reads a complete value from NVRAM.
-
-        Raises an `InvalidCommandResponse` error if the NVID doesn't exist.
-        """
-
-        # XXX: Some NVIDs don't really exist and Z-Stack behaves strangely with them
-        if nv_id == t.nvids.NwkNvIds.POLL_RATE_OLD16:
-            read_rsp = await self.request(
-                c.SYS.OSALNVRead.Req(Id=nv_id, Offset=0),
-                RspStatus=t.Status.SUCCESS,
-            )
-
-            return read_rsp.Value
-
-        # Every item has a length, even missing ones
-        length = (await self.request(c.SYS.OSALNVLength.Req(Id=nv_id))).ItemLen
-
-        if length == 0:
-            raise KeyError(f"NV item does not exist: {nv_id!r}")
-
-        data = b""
-
-        # Not all items can be read out due to security policies, though this can easily
-        # be bypassed for most
-        if (
-            nvids.is_secure_nvid(nv_id)
-            and self._capabilities & t.MTCapabilities.CAP_SAPI
-        ):
-            # The SAPI "ConfigId" is only 8 bits, which means some nvids are not
-            # able to read this way
-            if nv_id > 0xFF:
-                raise SecurityError(
-                    f"NV item cannot be read due to security constraints: {nv_id!r}"
-                )
-
-            # It is impossible to read an item that will not fit in a single response
-            assert length <= 247
-
-            read_rsp = await self.request(
-                c.SAPI.ZBReadConfiguration.Req(ConfigId=nv_id),
-                RspStatus=t.Status.SUCCESS,
-                RspConfigId=nv_id,
-            )
-
-            data = read_rsp.Value
-        else:
-            while len(data) < length:
-                read_rsp = await self.request(
-                    c.SYS.OSALNVReadExt.Req(Id=nv_id, Offset=len(data)),
-                    RspStatus=t.Status.SUCCESS,
-                )
-
-                data += read_rsp.Value
-
-        assert len(data) == length
-
-        return data
