@@ -2,6 +2,18 @@
 getting device info, getting NV info, subscribing callbacks, etc."""
 
 import zigpy_znp.types as t
+from zigpy_znp.znp.nib import PaddingByte
+
+
+class NodeRelation(t.enum_uint8):
+    PARENT = 0
+    CHILD_RFD = 1
+    CHILD_RFD_RX_IDLE = 2
+    CHILD_FFD = 3
+    CHILD_FFD_RX_IDLE = 4
+    NEIGHBOR = 5
+    OTHER = 6
+    NOTUSED = 0xFF
 
 
 class BindEntry(t.Struct):
@@ -31,16 +43,28 @@ class LinkInfo(t.Struct):
 class Device(t.Struct):
     shortAddr: t.NWK
     addrIdx: t.uint16_t
-    nodeRelation: t.uint8_t
+    nodeRelation: NodeRelation
     devStatus: t.uint8_t
     assocCnt: t.uint8_t
     age: t.uint8_t
     linkInfo: LinkInfo
+
+    PaddingByte1: PaddingByte
+    PaddingByte2: PaddingByte
+
     endDev: AgingEndDevice
+
+    PaddingByte3: PaddingByte
+    PaddingByte4: PaddingByte
+    PaddingByte5: PaddingByte
+
     timeoutCounter: t.uint32_t
-    keepaliveRcv: t.Bool
+    keepaliveRcv: t.uint8_t  # not a bool, can be 0xFF
     ctrl: t.uint8_t
 
+    PaddingByte6: PaddingByte
+    PaddingByte7: PaddingByte
+
 
 class Key(t.FixedList, item_type=t.uint8_t, length=42):
     pass
@@ -454,6 +478,32 @@ class Util(t.CommandsBase, subsystem=t.Subsystem.UTIL):
         ),
     )
 
+    # Z2M firmware: proxy call to AssocRemove
+    AssocRemove = t.CommandDef(
+        t.CommandType.SREQ,
+        0x63,
+        req_schema=(
+            t.Param("IEEE", t.EUI64, "Extended address of the device to remove"),
+        ),
+        rsp_schema=t.STATUS_SCHEMA,
+    )
+
+    # Z2M firmware: proxy call to AssocAddNew
+    AssocAdd = t.CommandDef(
+        t.CommandType.SREQ,
+        0x64,
+        req_schema=(
+            t.Param("NWK", t.NWK, "Short address of the device"),
+            t.Param("IEEE", t.EUI64, "Extended address of the device to add"),
+            t.Param(
+                "NodeRelation",
+                NodeRelation,
+                "Relation of the device to the coordinator",
+            ),
+        ),
+        rsp_schema=t.STATUS_SCHEMA,
+    )
+
     # a proxy call to zclGeneral_KeyEstablish_InitiateKeyEstablishment()
     ZCLKeyEstInitEst = t.CommandDef(
         t.CommandType.SREQ,
@@ -497,8 +547,7 @@ class Util(t.CommandsBase, subsystem=t.Subsystem.UTIL):
 
     # UTIL Callbacks
     # asynchronous request/response handshake
-    # XXX: This command is ambiguously defined.
-    #      We see an example of how it behaves.
+    # XXX: This command's request is completely identical to its response.
     # SyncReq = t.CommandDef(t.CommandType.AREQ, 0xE0)
 
     # RPC proxy indication for a ZCL_KEY_ESTABLISH_IND
