@@ -26,8 +26,8 @@ import zigpy_znp.config as conf
 import zigpy_znp.commands as c
 from zigpy_znp.api import ZNP
 from zigpy_znp.znp.nib import NIB, CC2531NIB, parse_nib
-from zigpy_znp.exceptions import InvalidCommandResponse
-from zigpy_znp.types.nvids import NwkNvIds
+from zigpy_znp.exceptions import CommandNotRecognized, InvalidCommandResponse
+from zigpy_znp.types.nvids import OsalNvIds
 from zigpy_znp.zigbee.zdo_converters import ZDO_CONVERTERS
 
 with warnings.catch_warnings():
@@ -43,10 +43,32 @@ with warnings.catch_warnings():
 ZDO_ENDPOINT = 0
 PROBE_TIMEOUT = 5  # seconds
 STARTUP_TIMEOUT = 5  # seconds
-ZDO_REQUEST_TIMEOUT = 10  # seconds
-DATA_CONFIRM_TIMEOUT = 5  # seconds
+ZDO_REQUEST_TIMEOUT = 15  # seconds
+DATA_CONFIRM_TIMEOUT = 8  # seconds
+DEVICE_JOIN_MAX_DELAY = 2  # seconds
 NETWORK_COMMISSIONING_TIMEOUT = 30  # seconds
 
+REQUEST_MAX_RETRIES = 5
+REQUEST_ERROR_RETRY_DELAY = 0.5  # second
+
+# Errors that go away on their own after waiting for a bit
+REQUEST_TRANSIENT_ERRORS = {
+    t.Status.BUFFER_FULL,
+    t.Status.MAC_CHANNEL_ACCESS_FAILURE,
+    t.Status.MAC_NO_RESOURCES,
+    t.Status.MEM_ERROR,
+    t.Status.NWK_TABLE_FULL,
+}
+
+REQUEST_ROUTING_ERRORS = {
+    t.Status.APS_NO_ACK,
+    t.Status.NWK_NO_ROUTE,
+    t.Status.MAC_NO_ACK,
+    t.Status.MAC_TRANSACTION_EXPIRED,
+}
+
+REQUEST_RETRYABLE_ERRORS = REQUEST_TRANSIENT_ERRORS | REQUEST_ROUTING_ERRORS
+
 DEFAULT_TC_LINK_KEY = t.TCLinkKey(
     ExtAddr=t.EUI64.convert("FF:FF:FF:FF:FF:FF:FF:FF"),  # global
     Key=t.KeyData(b"ZigBeeAlliance09"),
@@ -101,6 +123,9 @@ class ControllerApplication(zigpy.application.ControllerApplication):
         self._nib = NIB()
         self._network_key = None
         self._concurrent_requests_semaphore = None
+        self._currently_waiting_requests = 0
+        self._route_discovery_futures = {}
+        self._join_announce_tasks = {}
 
     ##################################################################
     # Implementation of the core zigpy ControllerApplication methods #
@@ -160,6 +185,11 @@ class ControllerApplication(zigpy.application.ControllerApplication):
 
         self._reconnect_task.cancel()
 
+        for f in self._route_discovery_futures.values():
+            f.cancel()
+
+        self._route_discovery_futures.clear()
+
         if self._znp is not None:
             self._znp.close()
             self._znp = None
@@ -190,22 +220,15 @@ class ControllerApplication(zigpy.application.ControllerApplication):
 
         self._bind_callbacks()
 
-        # XXX: To make sure we don't switch to the wrong device upon reconnect,
-        #      update our config to point to the last-detected port.
-        if self._config[conf.CONF_DEVICE][conf.CONF_DEVICE_PATH] == "auto":
-            self._config[conf.CONF_DEVICE][
-                conf.CONF_DEVICE_PATH
-            ] = self._znp._uart._transport.serial.name
-
         # Next, read out the NVRAM item that Zigbee2MQTT writes when it has configured
         # a device to make sure that our network settings will not be reset.
         if self.is_zstack_home_12:
-            configured_nv_item = NwkNvIds.HAS_CONFIGURED_ZSTACK1
+            configured_nv_item = OsalNvIds.HAS_CONFIGURED_ZSTACK1
         else:
-            configured_nv_item = NwkNvIds.HAS_CONFIGURED_ZSTACK3
+            configured_nv_item = OsalNvIds.HAS_CONFIGURED_ZSTACK3
 
         try:
-            configured_value = await self._znp.nvram_read(configured_nv_item)
+            configured_value = await self._znp.nvram.osal_read(configured_nv_item)
             is_configured = configured_value == ZSTACK_CONFIGURE_SUCCESS
         except KeyError:
             is_configured = False
@@ -274,6 +297,11 @@ class ControllerApplication(zigpy.application.ControllerApplication):
         async with async_timeout.timeout(STARTUP_TIMEOUT):
             await started_as_coordinator
 
+        # XXX: The CC2531 running Z-Stack Home 1.2 permanently permits joins on startup
+        # unless they are explicitly disabled. We can't fix this but we can disable them
+        # as early as possible to shrink the window of opportunity for unwanted joins.
+        await self.permit_ncp(time_s=0)
+
         # The CC2531 running Z-Stack Home 1.2 overrides the LED setting if it is changed
         # before the coordinator has started.
         if self.znp_config[conf.CONF_LED_MODE] is not None:
@@ -303,20 +331,7 @@ class ControllerApplication(zigpy.application.ControllerApplication):
         )
         self.zigpy_device.node_desc = node_descriptor_rsp.NodeDescriptor
 
-        # Get the currently active endpoints
-        endpoints = await self._znp.request_callback_rsp(
-            request=c.ZDO.ActiveEpReq.Req(DstAddr=0x0000, NWKAddrOfInterest=0x0000),
-            RspStatus=t.Status.SUCCESS,
-            callback=c.ZDO.ActiveEpRsp.Callback(partial=True),
-        )
-
-        # Clear them out
-        for endpoint in endpoints.ActiveEndpoints:
-            await self._znp.request(
-                c.AF.Delete.Req(Endpoint=endpoint), RspStatus=t.Status.SUCCESS
-            )
-
-        # And register our own
+        # Register our endpoints
         await self._register_endpoint(
             endpoint=1,
             profile_id=zigpy.profiles.zha.PROFILE_ID,
@@ -409,12 +424,12 @@ class ControllerApplication(zigpy.application.ControllerApplication):
             channels = t.Channels.from_channel_list([channel])
 
         # Delete any existing HAS_CONFIGURED_ZSTACK* NV items. One (or both) may fail.
-        await self._znp.nvram_delete(NwkNvIds.HAS_CONFIGURED_ZSTACK1)
-        await self._znp.nvram_delete(NwkNvIds.HAS_CONFIGURED_ZSTACK3)
+        await self._znp.nvram.osal_delete(OsalNvIds.HAS_CONFIGURED_ZSTACK1)
+        await self._znp.nvram.osal_delete(OsalNvIds.HAS_CONFIGURED_ZSTACK3)
 
         # Instruct Z-Stack to reset everything on the next boot
-        await self._znp.nvram_write(
-            NwkNvIds.STARTUP_OPTION,
+        await self._znp.nvram.osal_write(
+            OsalNvIds.STARTUP_OPTION,
             t.StartupOptions.ClearState | t.StartupOptions.ClearConfig,
         )
 
@@ -425,15 +440,15 @@ class ControllerApplication(zigpy.application.ControllerApplication):
         LOGGER.debug("Updating network settings")
 
         await self._write_stack_settings(reset_if_changed=False)
-        await self._znp.nvram_write(NwkNvIds.PANID, pan_id, create=True)
-        await self._znp.nvram_write(
-            NwkNvIds.APS_USE_EXT_PANID, extended_pan_id, create=True
+        await self._znp.nvram.osal_write(OsalNvIds.PANID, pan_id, create=True)
+        await self._znp.nvram.osal_write(
+            OsalNvIds.APS_USE_EXT_PANID, extended_pan_id, create=True
         )
-        await self._znp.nvram_write(NwkNvIds.PRECFGKEY, network_key, create=True)
-        await self._znp.nvram_write(
-            NwkNvIds.PRECFGKEYS_ENABLE, t.Bool(True), create=True
+        await self._znp.nvram.osal_write(OsalNvIds.PRECFGKEY, network_key, create=True)
+        await self._znp.nvram.osal_write(
+            OsalNvIds.PRECFGKEYS_ENABLE, t.Bool(True), create=True
         )
-        await self._znp.nvram_write(NwkNvIds.CHANLIST, channels, create=True)
+        await self._znp.nvram.osal_write(OsalNvIds.CHANLIST, channels, create=True)
 
         # Z-Stack Home 1.2 doesn't have the BDB subsystem
         if not self.is_zstack_home_12:
@@ -487,8 +502,8 @@ class ControllerApplication(zigpy.application.ControllerApplication):
                 ):
                     raise RuntimeError(f"Network formation failed: {commissioning_rsp}")
             else:
-                await self._znp.nvram_write(
-                    NwkNvIds.TCLK_SEED,
+                await self._znp.nvram.osal_write(
+                    OsalNvIds.TCLK_SEED,
                     value=DEFAULT_TC_LINK_KEY,
                     create=True,
                 )
@@ -509,7 +524,7 @@ class ControllerApplication(zigpy.application.ControllerApplication):
         # not appear to be any user-facing MT command to read this information.
         while True:
             try:
-                nib = parse_nib(await self._znp.nvram_read(NwkNvIds.NIB))
+                nib = parse_nib(await self._znp.nvram.osal_read(OsalNvIds.NIB))
 
                 LOGGER.debug("Current NIB is %s", nib)
 
@@ -523,11 +538,11 @@ class ControllerApplication(zigpy.application.ControllerApplication):
 
         # Create the NV item that keeps track of whether or not we're fully configured.
         if self.is_zstack_home_12:
-            configured_nv_item = NwkNvIds.HAS_CONFIGURED_ZSTACK1
+            configured_nv_item = OsalNvIds.HAS_CONFIGURED_ZSTACK1
         else:
-            configured_nv_item = NwkNvIds.HAS_CONFIGURED_ZSTACK3
+            configured_nv_item = OsalNvIds.HAS_CONFIGURED_ZSTACK3
 
-        await self._znp.nvram_write(
+        await self._znp.nvram.osal_write(
             configured_nv_item, ZSTACK_CONFIGURE_SUCCESS, create=True
         )
 
@@ -567,10 +582,10 @@ class ControllerApplication(zigpy.application.ControllerApplication):
         expect_reply=True,
         use_ieee=False,
     ) -> typing.Tuple[t.Status, str]:
-        tx_options = c.af.TransmitOptions.RouteDiscovery
+        tx_options = c.af.TransmitOptions.SUPPRESS_ROUTE_DISC_NETWORK
 
         if expect_reply:
-            tx_options |= c.af.TransmitOptions.APSAck
+            tx_options |= c.af.TransmitOptions.ACK_REQUEST
 
         if use_ieee:
             destination = t.AddrModeAddress(mode=t.AddrMode.IEEE, address=device.ieee)
@@ -632,7 +647,7 @@ class ControllerApplication(zigpy.application.ControllerApplication):
         return await self._send_request(
             dst_addr=t.AddrModeAddress(mode=t.AddrMode.Group, address=group_id),
             dst_ep=src_ep,
-            src_ep=src_ep,  # not actually used?
+            src_ep=src_ep,
             profile=profile,
             cluster=cluster,
             sequence=sequence,
@@ -641,26 +656,33 @@ class ControllerApplication(zigpy.application.ControllerApplication):
             data=data,
         )
 
-    async def force_remove(self, device) -> None:
+    async def force_remove(self, device: zigpy.device.Device) -> None:
         """
-        Forcibly removes a direct child from the network.
+        Attempts to forcibly remove a device from the network.
+        Z-Stack does not expose any additional ways to to do this.
         """
 
-        async with self._limit_concurrency():
-            leave_rsp = await self._znp.request_callback_rsp(
-                request=c.ZDO.MgmtLeaveReq.Req(
-                    DstAddr=0x0000,  # We handle it
-                    IEEE=device.ieee,
-                    RemoveChildren_Rejoin=c.zdo.LeaveOptions.NONE,
-                ),
-                RspStatus=t.Status.SUCCESS,
-                callback=c.ZDO.MgmtLeaveRsp.Callback(Src=0x0000, partial=True),
+        try:
+            async with self._limit_concurrency():
+                await self._znp.request_callback_rsp(
+                    request=c.ZDO.MgmtLeaveReq.Req(
+                        DstAddr=0x0000,
+                        IEEE=device.ieee,
+                        RemoveChildren_Rejoin=c.zdo.LeaveOptions.NONE,
+                    ),
+                    RspStatus=t.Status.SUCCESS,
+                    callback=c.ZDO.MgmtLeaveRsp.Callback(
+                        Src=0x0000,
+                        partial=True,
+                    ),
+                )
+        except Exception as e:
+            LOGGER.warning(
+                "Failed to remove device %s from coordinator: %s",
+                device.ieee,
+                e,
             )
 
-        assert leave_rsp.Status == t.ZDOStatus.SUCCESS
-
-        # TODO: see what happens when we forcibly remove a device that isn't our child
-
     async def permit_ncp(self, time_s: int) -> None:
         """
         Permits new devices to join the network *only through us*.
@@ -678,7 +700,7 @@ class ControllerApplication(zigpy.application.ControllerApplication):
                 TCSignificance=1,
             ),
             RspStatus=t.Status.SUCCESS,
-            callback=c.ZDO.MgmtPermitJoinRsp.Callback(partial=True),
+            callback=c.ZDO.MgmtPermitJoinRsp.Callback(Src=0x0000, partial=True),
         )
 
         if response.Status != t.Status.SUCCESS:
@@ -733,7 +755,7 @@ class ControllerApplication(zigpy.application.ControllerApplication):
 
         self._znp = None
 
-        # exc=None means that the connection was closed
+        # exc=None means the connection was closed
         if exc is None:
             LOGGER.debug("Connection was purposefully closed. Not reconnecting.")
             return
@@ -803,13 +825,18 @@ class ControllerApplication(zigpy.application.ControllerApplication):
 
         LOGGER.info("ZDO device announce: %s", msg)
 
-        try:
-            device = self.get_device(ieee=msg.IEEE)
-        except KeyError:
-            LOGGER.warning(
-                "Received a ZDO message from an unknown device: %s", msg.IEEE
-            )
-            return
+        # Cancel an existing join timer so we don't double announce
+        if msg.IEEE in self._join_announce_tasks:
+            self._join_announce_tasks.pop(msg.IEEE).cancel()
+
+        # Sometimes devices change their NWK when announcing so re-join it.
+        self.handle_join(
+            nwk=msg.NWK,
+            ieee=msg.IEEE,
+            parent_nwk=None,
+        )
+
+        device = self.get_device(ieee=msg.IEEE)
 
         # We turn this back into a ZDO message and let zigpy handle it
         self._receive_zdo_message(
@@ -823,11 +850,29 @@ class ControllerApplication(zigpy.application.ControllerApplication):
 
     def on_zdo_tc_device_join(self, msg: c.ZDO.TCDevInd.Callback) -> None:
         """
-        ZDO trust center device join callback
+        ZDO trust center device join callback.
         """
 
         LOGGER.info("TC device join: %s", msg)
-        self.handle_join(nwk=msg.SrcNwk, ieee=msg.SrcIEEE, parent_nwk=msg.ParentNwk)
+
+        # Perform route discovery (just in case) when a device joins the network so that
+        # we can begin initialization as soon as possible.
+        asyncio.create_task(self._discover_route(msg.SrcNwk))
+
+        if msg.SrcIEEE in self._join_announce_tasks:
+            self._join_announce_tasks.pop(msg.SrcIEEE).cancel()
+
+        # Some devices really don't like zigpy beginning its initialization process
+        # before the device has announced itself. Wait a second or two before calling
+        # `handle_join`, just in case the device announces itself first.
+        self._join_announce_tasks[msg.SrcIEEE] = asyncio.get_running_loop().call_later(
+            DEVICE_JOIN_MAX_DELAY,
+            lambda: self.handle_join(
+                nwk=msg.SrcNwk,
+                ieee=msg.SrcIEEE,
+                parent_nwk=msg.ParentNwk,
+            ),
+        )
 
     def on_zdo_device_leave(self, msg: c.ZDO.LeaveInd.Callback) -> None:
         LOGGER.info("ZDO device left: %s", msg)
@@ -900,30 +945,30 @@ class ControllerApplication(zigpy.application.ControllerApplication):
 
         # It's better to be explicit than rely on the NVRAM defaults
         settings = {
-            NwkNvIds.LOGICAL_TYPE: t.DeviceLogicalType.Coordinator,
+            OsalNvIds.LOGICAL_TYPE: t.DeviceLogicalType.Coordinator,
             # Source routing
-            NwkNvIds.CONCENTRATOR_ENABLE: t.Bool(True),
-            NwkNvIds.CONCENTRATOR_DISCOVERY: t.uint8_t(120),
-            NwkNvIds.CONCENTRATOR_RC: t.Bool(True),
-            NwkNvIds.SRC_RTG_EXPIRY_TIME: t.uint8_t(255),
-            NwkNvIds.NWK_CHILD_AGE_ENABLE: t.Bool(False),
+            OsalNvIds.CONCENTRATOR_ENABLE: t.Bool(True),
+            OsalNvIds.CONCENTRATOR_DISCOVERY: t.uint8_t(120),
+            OsalNvIds.CONCENTRATOR_RC: t.Bool(True),
+            OsalNvIds.SRC_RTG_EXPIRY_TIME: t.uint8_t(255),
+            OsalNvIds.NWK_CHILD_AGE_ENABLE: t.Bool(False),
             # Default is 20 in Z-Stack 3.0.1, 30 in Z-Stack 3/4
-            NwkNvIds.BCAST_DELIVERY_TIME: t.uint8_t(30),
+            OsalNvIds.BCAST_DELIVERY_TIME: t.uint8_t(30),
             # We want to receive all ZDO callbacks to proxy them back to zigpy
-            NwkNvIds.ZDO_DIRECT_CB: t.Bool(True),
+            OsalNvIds.ZDO_DIRECT_CB: t.Bool(True),
         }
 
         any_changed = False
 
         for nvid, value in settings.items():
             try:
-                current_value = await self._znp.nvram_read(nvid)
+                current_value = await self._znp.nvram.osal_read(nvid)
             except InvalidCommandResponse:
                 current_value = None
 
             # There is no point in issuing a write if the value will not change
             if current_value != value.serialize():
-                await self._znp.nvram_write(nvid, value)
+                await self._znp.nvram.osal_write(nvid, value)
                 any_changed = True
 
         if reset_if_changed and any_changed:
@@ -935,25 +980,31 @@ class ControllerApplication(zigpy.application.ControllerApplication):
         """
         Async context manager that prevents devices from being overwhelmed by requests.
         Mainly a thin wrapper around `asyncio.Semaphore` that logs when it has to wait.
-
-        TODO: it would be better to also delay requests in response to `TABLE_FULL`.
         """
 
         start_time = time.time()
         was_locked = self._concurrent_requests_semaphore.locked()
 
         if was_locked:
-            LOGGER.debug("Max concurrency reached, delaying requests")
+            self._currently_waiting_requests += 1
+            LOGGER.debug(
+                "Max concurrency reached, delaying requests (%s enqueued)",
+                self._currently_waiting_requests,
+            )
 
-        async with self._concurrent_requests_semaphore:
+        try:
+            async with self._concurrent_requests_semaphore:
+                if was_locked:
+                    LOGGER.debug(
+                        "Previously delayed request is now running, "
+                        "delayed by %0.2f seconds",
+                        time.time() - start_time,
+                    )
+
+                yield
+        finally:
             if was_locked:
-                LOGGER.debug(
-                    "Previously delayed request is now running, "
-                    "delayed by %0.2f seconds",
-                    time.time() - start_time,
-                )
-
-            yield
+                self._currently_waiting_requests -= 1
 
     def _receive_zdo_message(
         self,
@@ -1057,9 +1108,9 @@ class ControllerApplication(zigpy.application.ControllerApplication):
 
     async def _load_device_info(self):
         # Parsing the NIB struct gives us access to low-level info, like the channel
-        self._nib = parse_nib(await self._znp.nvram_read(NwkNvIds.NIB))
+        self._nib = parse_nib(await self._znp.nvram.osal_read(OsalNvIds.NIB))
         self._network_key, _ = t.KeyData.deserialize(
-            await self._znp.nvram_read(NwkNvIds.PRECFGKEY)
+            await self._znp.nvram.osal_read(OsalNvIds.PRECFGKEY)
         )
 
         LOGGER.debug("Parsed NIB: %s", self._nib)
@@ -1084,7 +1135,8 @@ class ControllerApplication(zigpy.application.ControllerApplication):
         if dst_ep == ZDO_ENDPOINT:
             return ZDO_ENDPOINT
 
-        candidates = []
+        # Always fall back to endpoint 1
+        candidates = [1]
 
         for ep_id, endpoint in self.zigpy_device.endpoints.items():
             if ep_id == ZDO_ENDPOINT:
@@ -1102,22 +1154,15 @@ class ControllerApplication(zigpy.application.ControllerApplication):
             # if we don't find anything better
             candidates.append(endpoint.endpoint_id)
 
-        if not candidates:
-            raise ValueError(
-                f"Could not pick endpoint for dst_ep={dst_ep},"
-                f" profile={profile}, and cluster={cluster}"
-            )
-
-        # XXX: pick the first one?
-        return candidates[0]
+        return candidates[-1]
 
     async def _send_zdo_request(
         self, dst_addr, dst_ep, src_ep, cluster, sequence, options, radius, data
     ):
         """
-        Zigpy doesn't send ZDO requests via TI's ZDO_* MT commands,
-        so it will never receive a reply because ZNP intercepts ZDO replies, never
-        sends a DataConfirm, and instead replies with one of its ZDO_* MT responses.
+        Zigpy doesn't send ZDO requests via TI's ZDO_* MT commands, so it will never
+        receive a reply because ZNP intercepts ZDO replies, never sends a DataConfirm,
+        and instead replies with one of its ZDO_* MT responses.
 
         This method translates the ZDO_* MT response into one zigpy can handle.
         """
@@ -1143,21 +1188,21 @@ class ControllerApplication(zigpy.application.ControllerApplication):
         zdo_args, _ = list_deserialize(data, field_types)
         zdo_kwargs = dict(zip(field_names, zdo_args))
 
-        device = self.get_device(nwk=dst_addr.address)
+        # TODO: Check out `ZDO.MsgCallbackRegister`
 
         if cluster not in ZDO_CONVERTERS:
             LOGGER.error(
                 "ZDO converter for cluster %s has not been implemented!"
                 " Please open a GitHub issue and attach a debug log:"
-                " https://github.com/zha-ng/zigpy-znp/issues/new",
+                " https://github.com/zigpy/zigpy-znp/issues/new",
                 cluster,
             )
-            return t.Status.FAILURE, "No ZDO converter"
+            raise RuntimeError("No ZDO converter")
 
         # Call the converter with the ZDO request's kwargs
         req_factory, rsp_factory, zdo_rsp_factory = ZDO_CONVERTERS[cluster]
-        request = req_factory(dst_addr.address, device, **zdo_kwargs)
-        callback = rsp_factory(dst_addr.address)
+        request = req_factory(dst_addr, **zdo_kwargs)
+        callback = rsp_factory(dst_addr)
 
         LOGGER.debug(
             "Intercepted AP ZDO request %s(%s) and replaced with %s",
@@ -1166,22 +1211,147 @@ class ControllerApplication(zigpy.application.ControllerApplication):
             request,
         )
 
-        try:
-            async with self._limit_concurrency():
-                async with async_timeout.timeout(ZDO_REQUEST_TIMEOUT):
-                    response = await self._znp.request_callback_rsp(
-                        request=request, RspStatus=t.Status.SUCCESS, callback=callback
+        # The coordinator responds to broadcasts
+        if dst_addr.mode == t.AddrMode.Broadcast:
+            callback = callback.replace(Src=0x0000)
+
+        async with async_timeout.timeout(ZDO_REQUEST_TIMEOUT):
+            response = await self._znp.request_callback_rsp(
+                request=request, RspStatus=t.Status.SUCCESS, callback=callback
+            )
+
+        # We should only send zigpy unicast responses
+        if dst_addr.mode == t.AddrMode.NWK:
+            zdo_rsp_cluster, zdo_response_kwargs = zdo_rsp_factory(response)
+
+            self._receive_zdo_message(
+                cluster=zdo_rsp_cluster,
+                tsn=sequence,
+                sender=self.get_device(nwk=dst_addr.address),
+                **zdo_response_kwargs,
+            )
+
+        return response
+
+    async def _send_request_raw(
+        self,
+        dst_addr,
+        dst_ep,
+        src_ep,
+        profile,
+        cluster,
+        sequence,
+        options,
+        radius,
+        data,
+        *,
+        relays=None,
+    ):
+        """
+        Used by `request`/`mrequest`/`broadcast` to send a request.
+        Picks the correct request sending mechanism and fixes endpoint information.
+        """
+
+        # ZDO requests must be handled by the translation layer, since Z-Stack will
+        # "steal" the responses
+        if dst_ep == ZDO_ENDPOINT:
+            return await self._send_zdo_request(
+                dst_addr, dst_ep, src_ep, cluster, sequence, options, radius, data
+            )
+
+        # Zigpy just sets src == dst, which doesn't work for devices with many endpoints
+        # We pick ours based on the registered endpoints.
+        src_ep = self._find_endpoint(dst_ep=dst_ep, profile=profile, cluster=cluster)
+
+        if relays is None:
+            request = c.AF.DataRequestExt.Req(
+                DstAddrModeAddress=dst_addr,
+                DstEndpoint=dst_ep,
+                DstPanId=0x0000,
+                SrcEndpoint=src_ep,
+                ClusterId=cluster,
+                TSN=sequence,
+                Options=options,
+                Radius=radius,
+                Data=data,
+            )
+        else:
+            request = c.AF.DataRequestSrcRtg.Req(
+                DstAddr=dst_addr.address,
+                DstEndpoint=dst_ep,
+                SrcEndpoint=src_ep,
+                ClusterId=cluster,
+                TSN=sequence,
+                Options=options,
+                Radius=radius,
+                SourceRoute=relays,  # force the packet to go through specific parents
+                Data=data,
+            )
+
+        if dst_addr.mode == t.AddrMode.Broadcast:
+            # Broadcasts will not receive a confirmation but they still take time
+            # and use up concurrency slots
+            response = await self._znp.request(
+                request=request, RspStatus=t.Status.SUCCESS
+            )
+
+            await asyncio.sleep(0.1 * self._nib.BroadcastDeliveryTime)
+        else:
+            async with async_timeout.timeout(DATA_CONFIRM_TIMEOUT):
+                # Shield from cancellation to prevent requests that time out
+                # in higher layers from missing expected responses
+                response = await asyncio.shield(
+                    self._znp.request_callback_rsp(
+                        request=request,
+                        RspStatus=t.Status.SUCCESS,
+                        callback=c.AF.DataConfirm.Callback(
+                            partial=True,
+                            TSN=sequence,
+                            # XXX: can this ever not match?
+                            # Endpoint=src_ep,
+                        ),
                     )
-        except InvalidCommandResponse as e:
-            raise DeliveryError(f"Could not send command: {e.response.Status}") from e
+                )
 
-        zdo_rsp_cluster, zdo_response_kwargs = zdo_rsp_factory(response)
+                # Both the callback and the response can have an error status
+                if response.Status != t.Status.SUCCESS:
+                    raise InvalidCommandResponse(
+                        f"Unsuccessful request status code: {response.Status!r}",
+                        response,
+                    )
 
-        self._receive_zdo_message(
-            cluster=zdo_rsp_cluster, tsn=sequence, sender=device, **zdo_response_kwargs
-        )
+        return response
 
-        return response.Status, "Request sent successfully"
+    async def _discover_route(self, nwk: t.NWK) -> None:
+        """
+        Instructs the coordinator to re-discover routes to the provided NWK.
+        Runs concurrently and at most once per NWK, even if called multiple times.
+        """
+
+        # Route discovery with Z-Stack 1.2 and Z-Stack 3.0.2 on the CC2531 doesn't
+        # appear to work very well (Z2M#2901)
+        if self.is_cc2531:
+            return
+
+        if nwk in self._route_discovery_futures:
+            return await self._route_discovery_futures[nwk]
+
+        future = asyncio.get_running_loop().create_future()
+        self._route_discovery_futures[nwk] = future
+
+        try:
+            await self._znp.request(
+                c.ZDO.ExtRouteDisc.Req(
+                    Dst=nwk,
+                    Options=c.zdo.RouteDiscoveryOptions.UNICAST,
+                    Radius=30,
+                ),
+            )
+
+            await asyncio.sleep(0.1 * self._nib.RouteDiscoveryTime)
+        finally:
+            future.set_result(True)
+            del self._route_discovery_futures[nwk]
 
     async def _send_request(
         self,
@@ -1196,52 +1366,180 @@ class ControllerApplication(zigpy.application.ControllerApplication):
         data,
     ) -> typing.Tuple[t.Status, str]:
         """
-        Used by `request`/`mrequest`/`broadcast` to send a request.
-        Picks the correct request sending mechanism and fixes endpoint information.
+        Fault-tolerant wrapper around `_send_request_raw` that transparently attempts to
+        repair routes and contact the device through other methods when Z-Stack errors
+        are encountered.
         """
 
-        if dst_ep == ZDO_ENDPOINT and not (
-            cluster == ZDOCmd.Mgmt_Permit_Joining_req
-            and dst_addr.mode == t.AddrMode.Broadcast
-        ):
-            return await self._send_zdo_request(
-                dst_addr, dst_ep, src_ep, cluster, sequence, options, radius, data
-            )
-
-        # Zigpy just sets src == dst, which doesn't work for devices with many endpoints
-        # We pick ours based on the registered endpoints.
-        src_ep = self._find_endpoint(dst_ep=dst_ep, profile=profile, cluster=cluster)
-
-        request = c.AF.DataRequestExt.Req(
-            DstAddrModeAddress=dst_addr,
-            DstEndpoint=dst_ep,
-            DstPanId=0x0000,
-            SrcEndpoint=src_ep,
-            ClusterId=cluster,
-            TSN=sequence,
-            Options=options,
-            Radius=radius,
-            Data=data,
-        )
-
-        if dst_addr.mode == t.AddrMode.Broadcast:
-            # We won't always get a data confirmation
-            response = await self._znp.request(
-                request=request, RspStatus=t.Status.SUCCESS
-            )
+        if dst_addr.mode == t.AddrMode.NWK:
+            device = self.get_device(nwk=dst_addr.address)
+        elif dst_addr.mode == t.AddrMode.IEEE:
+            device = self.get_device(ieee=dst_addr.address)
         else:
-            async with self._limit_concurrency():
-                async with async_timeout.timeout(DATA_CONFIRM_TIMEOUT):
-                    response = await self._znp.request_callback_rsp(
-                        request=request,
-                        RspStatus=t.Status.SUCCESS,
-                        callback=c.AF.DataConfirm.Callback(partial=True, TSN=sequence),
-                    )
+            device = None
 
-                LOGGER.debug("Received a data request confirmation: %s", response)
+        status = None
+        response = None
+        association = None
+        force_relays = None
+
+        tried_assoc_remove = False
+        tried_route_discovery = False
+        tried_disable_route_discovery_suppression = False
+        tried_last_good_route = False
+
+        # Don't release the concurrency-limiting semaphore until we are done trying.
+        # There is no point in allowing requests to take turns getting buffer errors.
+        try:
+            async with self._limit_concurrency():
+                for attempt in range(REQUEST_MAX_RETRIES):
+                    try:
+                        # ZDO requests do not generate `AF.DataConfirm` messages
+                        # indicating that a route is missing so we need to explicitly
+                        # check for one.
+                        if (
+                            dst_ep == ZDO_ENDPOINT
+                            and dst_addr.mode == t.AddrMode.NWK
+                            and dst_addr.address != 0x0000
+                        ):
+                            route_status = await self._znp.request(
+                                c.ZDO.ExtRouteChk.Req(
+                                    Dst=dst_addr.address,
+                                    RtStatus=c.zdo.RouteStatus.ACTIVE,
+                                    Options=(
+                                        c.zdo.RouteOptions.MTO_ROUTE
+                                        | c.zdo.RouteOptions.NO_ROUTE_CACHE
+                                    ),
+                                )
+                            )
+
+                            if route_status.Status != c.zdo.RoutingStatus.SUCCESS:
+                                await self._discover_route(dst_addr.address)
+
+                        response = await self._send_request_raw(
+                            dst_addr=dst_addr,
+                            dst_ep=dst_ep,
+                            src_ep=src_ep,
+                            profile=profile,
+                            cluster=cluster,
+                            sequence=sequence,
+                            options=options,
+                            radius=radius,
+                            data=data,
+                            relays=force_relays,
+                        )
+                        break
+                    except InvalidCommandResponse as e:
+                        status = e.response.Status
+
+                        if status not in REQUEST_RETRYABLE_ERRORS:
+                            raise
+
+                        # We cannot do anything but retry if the error is transient or
+                        # we are not sending a unicast request. Retry at least once.
+                        if (
+                            status in REQUEST_TRANSIENT_ERRORS
+                            or attempt == 0
+                            or device is None
+                        ):
+                            LOGGER.debug(
+                                "Request failed (%s), retry attempt %s of %s",
+                                e,
+                                attempt + 1,
+                                REQUEST_MAX_RETRIES,
+                            )
+                            await asyncio.sleep(3 * REQUEST_ERROR_RETRY_DELAY)
+                            continue
+
+                        # If we can't contact the device by forcing a specific route,
+                        # there is not point in trying this more than once.
+                        if tried_last_good_route and force_relays is not None:
+                            force_relays = None
+
+                        # Child aging is disabled so if a child switches parents from
+                        # the coordinator to another router, we will not be able to
+                        # re-discover a route to it. We have to manually drop the child
+                        # to do this.
+                        if (
+                            status == t.Status.MAC_TRANSACTION_EXPIRED
+                            and association is None
+                            and not tried_assoc_remove
+                        ):
+                            # XXX: do we use NWK or IEEE?
+                            association = await self._znp.request(
+                                c.Util.AssocGetWithAddress.Req(
+                                    IEEE=device.ieee,
+                                    NWK=device.nwk,
+                                )
+                            )
+
+                            if (
+                                association.Device.nodeRelation
+                                != c.util.NodeRelation.NOTUSED
+                            ):
+                                try:
+                                    await self._znp.request(
+                                        c.Util.AssocRemove.Req(IEEE=device.ieee)
+                                    )
+                                    tried_assoc_remove = True
+
+                                    # Route discovery must be performed right after
+                                    await self._discover_route(device.nwk)
+                                except CommandNotRecognized:
+                                    LOGGER.debug(
+                                        "The UTIL.AssocRemove command is available only"
+                                        " in Z-Stack 3 releases built after 20201017"
+                                    )
+                        elif not tried_route_discovery:
+                            # If that doesn't work, try re-discovering the route.
+                            # While we can in theory poll and wait until it is fixed,
+                            # letting the retry mechanism deal with it simpler.
+                            await self._discover_route(device.nwk)
+                            tried_route_discovery = True
+                        elif not tried_disable_route_discovery_suppression:
+                            # Disable route discovery suppression. This appears to
+                            # generate a bit more network traffic.
+                            options &= ~c.af.TransmitOptions.SUPPRESS_ROUTE_DISC_NETWORK
+                            tried_disable_route_discovery_suppression = True
+                        elif (
+                            not tried_last_good_route
+                            and device is not None
+                            and device.relays is not None
+                        ):
+                            # `ZDO.SrcRtgInd` callbacks tell us the last path taken by
+                            # messages from the device back to the coordinator. Sending
+                            # packets backwards via this same route may work.
+                            force_relays = device.relays[::-1]
+                            tried_last_good_route = True
+
+                        LOGGER.debug(
+                            "Request failed (%s), retry attempt %s of %s",
+                            e,
+                            attempt + 1,
+                            REQUEST_MAX_RETRIES,
+                        )
+
+                        # We've tried everything already so at this point just wait
+                        await asyncio.sleep(REQUEST_ERROR_RETRY_DELAY)
+                else:
+                    raise DeliveryError(
+                        f"Request failed after {REQUEST_MAX_RETRIES} attempts:"
+                        f" {status!r}"
+                    )
+        finally:
+            # We *must* re-add the device association if we previously removed it but
+            # the request still failed. Otherwise, it may be a direct child and we will
+            # not be able to find it again.
+            if tried_assoc_remove and response is None:
+                await self._znp.request(
+                    c.Util.AssocAdd.Req(
+                        NWK=device.nwk,
+                        IEEE=device.ieee,
+                        NodeRelation=association.Device.nodeRelation,
+                    )
+                )
 
         if response.Status != t.Status.SUCCESS:
-            LOGGER.warning("Failed to send request %s: %s", request, response.Status)
-            return response.Status, "Invalid response status"
+            return response.Status, "Failed to send request"
 
-        return response.Status, "Request sent successfully"
+        return response.Status, "Sent request successfully"
